"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const cli_ux_1 = require("cli-ux");
const FormData = require("form-data");
const fs = require("fs");
const node_fetch_1 = require("node-fetch");
const misc_1 = require("../misc");
class Deploy extends command_1.Command {
    async run() {
        const { flags } = this.parse(Deploy);
        cli_ux_1.default.action.start(`${chalk_1.default.green('packaging')} your app`);
        const fd = new misc_1.FolderDetails(this);
        try {
            await fd.readInfoFile();
        }
        catch (e) {
            this.error(e.message);
            return;
        }
        const compiler = new misc_1.AppCompiler(this, fd);
        const report = compiler.logDiagnostics();
        if (!report.isValid && !flags.force) {
            this.error('TypeScript compiler error(s) occurred');
            this.exit(1);
            return;
        }
        const packager = new misc_1.AppPackager(this, fd);
        const zipName = await packager.zipItUp();
        cli_ux_1.default.action.stop('packaged!');
        if (!flags.url) {
            flags.url = await cli_ux_1.default.prompt('What is the server\'s url (include https)?');
        }
        if (!flags.username) {
            flags.username = await cli_ux_1.default.prompt('What is the username?');
        }
        if (!flags.password) {
            flags.password = await cli_ux_1.default.prompt('And, what is the password?', { type: 'hide' });
        }
        cli_ux_1.default.action.start(`${chalk_1.default.green('deploying')} your app`);
        const data = new FormData();
        data.append('app', fs.createReadStream(fd.mergeWithFolder(zipName)));
        await this.asyncSubmitData(data, flags, fd);
        cli_ux_1.default.action.stop('deployed!');
    }
    // tslint:disable:promise-function-async
    async asyncSubmitData(data, flags, fd) {
        const authResult = await node_fetch_1.default(this.normalizeUrl(flags.url, '/api/v1/login'), {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({ username: flags.username, password: flags.password }),
        }).then((res) => res.json());
        if (authResult.status === 'error' || !authResult.data) {
            throw new Error('Invalid username and password');
        }
        let endpoint = '/api/apps';
        if (flags.update) {
            endpoint += `/${fd.info.id}`;
        }
        const deployResult = await node_fetch_1.default(this.normalizeUrl(flags.url, endpoint), {
            method: 'POST',
            headers: {
                'X-Auth-Token': authResult.data.authToken,
                'X-User-Id': authResult.data.userId,
            },
            body: data,
        }).then((res) => res.json());
        if (deployResult.status === 'error') {
            throw new Error('Unknown error occurred while deploying');
        }
        else if (!deployResult.success) {
            throw new Error(`Deployment error: ${deployResult.error}`);
        }
        if (deployResult.compilerErrors && deployResult.compilerErrors.length > 0) {
            throw new Error(`Deployment compiler errors: \n${JSON.stringify(deployResult.compilerErrors, null, 2)}`);
        }
    }
    // expects the `path` to start with the /
    normalizeUrl(url, path) {
        return url.replace(/\/$/, '') + path;
    }
}
Deploy.description = 'allows deploying an App to a server';
Deploy.flags = {
    help: command_1.flags.help({ char: 'h' }),
    // flag with no value (-f, --force)
    force: command_1.flags.boolean({ char: 'f', description: 'forcefully deploy the App, ignores lint & TypeScript errors' }),
    update: command_1.flags.boolean({ description: 'updates the app, instead of creating' }),
    url: command_1.flags.string({ description: 'where the App should be deployed to' }),
    username: command_1.flags.string({ char: 'u', dependsOn: ['url'], description: 'username to authenticate with' }),
    password: command_1.flags.string({ char: 'p', dependsOn: ['username'], description: 'password of the user' }),
};
exports.default = Deploy;
//# sourceMappingURL=deploy.js.map