"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const cli_ux_1 = require("cli-ux");
const FormData = require("form-data");
const fs = require("fs");
const fuzzy = require("fuzzy");
const inquirer = require("inquirer");
const node_fetch_1 = require("node-fetch");
const misc_1 = require("../misc");
class Submit extends command_1.Command {
    async run() {
        inquirer.registerPrompt('checkbox-plus', require('inquirer-checkbox-plus-prompt'));
        const { flags } = this.parse(Submit);
        //#region app packaging
        cli_ux_1.default.action.start(`${chalk_1.default.green('packaging')} your app`);
        const fd = new misc_1.FolderDetails(this);
        try {
            await fd.readInfoFile();
        }
        catch (e) {
            this.error(e.message);
            return;
        }
        const compiler = new misc_1.AppCompiler(this, fd);
        const report = compiler.logDiagnostics();
        if (!report.isValid) {
            this.error('TypeScript compiler error(s) occurred');
            this.exit(1);
            return;
        }
        const packager = new misc_1.AppPackager(this, fd);
        const zipName = await packager.zipItUp();
        cli_ux_1.default.action.stop('packaged!');
        //#endregion
        //#region fetching categories
        cli_ux_1.default.action.start(`${chalk_1.default.green('fetching')} the available categories`);
        const categories = await misc_1.VariousUtils.fetchCategories();
        cli_ux_1.default.action.stop('fetched!');
        //#endregion
        //#region asking for information
        if (!flags.email) {
            const result = await inquirer.prompt([{
                    type: 'input',
                    name: 'email',
                    message: 'What is the publisher\'s email address?',
                    validate: (answer) => {
                        const regex = /^[^@\s]+@[^@\s]+\.[^@\s]+$/g;
                        return regex.test(answer);
                    },
                }]);
            flags.email = result.email;
        }
        if (typeof flags.update === 'undefined') {
            const isNewApp = await inquirer.prompt([{
                    type: 'confirm',
                    name: 'isNew',
                    message: 'Is this a new App?',
                    default: true,
                }]);
            flags.update = !isNewApp.isNew;
        }
        let selectedCategories = new Array();
        if (flags.categories) {
            selectedCategories = flags.categories.split(',');
        }
        else {
            const result = await inquirer.prompt([{
                    type: 'checkbox-plus',
                    name: 'categories',
                    message: 'Please select the categories which apply to this App?',
                    pageSize: 10,
                    highlight: true,
                    searchable: true,
                    validate: (answer) => {
                        if (answer.length === 0) {
                            return 'You must choose at least one color.';
                        }
                        return true;
                    },
                    // tslint:disable:promise-function-async
                    source: (answersSoFar, input) => {
                        input = input || '';
                        return new Promise((resolve) => {
                            const fuzzyResult = fuzzy.filter(input, categories, {
                                extract: (item) => item.name,
                            });
                            const data = fuzzyResult.map((element) => {
                                return element.original;
                            });
                            resolve(data);
                        });
                    },
                }]);
            selectedCategories = result.categories;
        }
        const confirmSubmitting = await inquirer.prompt([{
                type: 'confirm',
                name: 'submit',
                message: 'Are you ready to submit?',
                default: false,
            }]);
        if (!confirmSubmitting.submit) {
            return;
        }
        //#endregion
        cli_ux_1.default.action.start(`${chalk_1.default.green('submitting')} your app`);
        const data = new FormData();
        data.append('app', fs.createReadStream(fd.mergeWithFolder(zipName)));
        data.append('email', flags.email);
        data.append('categories', JSON.stringify(selectedCategories));
        await this.asyncSubmitData(data, flags, fd);
        cli_ux_1.default.action.stop('submitted!');
    }
    // tslint:disable:promise-function-async
    async asyncSubmitData(data, flags, fd) {
        let url = 'https://marketplace.rocket.chat/v1/apps';
        if (flags.update) {
            url += `/${fd.info.id}`;
        }
        const res = await node_fetch_1.default(url, {
            method: 'POST',
            body: data,
        });
        if (res.status !== 200) {
            const result = await res.json();
            throw new Error(`Failed to submit the App. Error code ${result.code}: ${result.error}`);
        }
        else {
            return res.json();
        }
    }
}
Submit.description = 'submits an App to the Marketplace for review';
Submit.flags = {
    help: command_1.flags.help({ char: 'h' }),
    // In the future, we will allow people to have their own marketplace instances
    // url: flags.string({ description: 'which marketplace should be used' }),
    update: command_1.flags.boolean({ description: 'submits an update instead of creating one' }),
    email: command_1.flags.string({ char: 'e', description: 'the email of the publisher account' }),
    categories: command_1.flags.string({
        char: 'c',
        dependsOn: ['email'],
        description: 'a comma separated list of the categories for the App',
    }),
};
exports.default = Submit;
//# sourceMappingURL=submit.js.map